import numpy as np
from diffusers import StableDiffusionInpaintPipeline, StableDiffusionPipeline, EulerDiscreteScheduler
import torch
from fastmodel.backend.util.img_util import img_preprocess_sd_inpaint
from PIL import Image
from fastmodel.backend.constant.biz_constants import app_abs_path


def inference_sd_inpaint(image: str, mask: str, prompt: str, device: str, out_img: str):
    print(image, mask, prompt)
    pipe = StableDiffusionInpaintPipeline.from_pretrained(
        "stabilityai/stable-diffusion-2-inpainting",
        torch_dtype=torch.float,
    )
    pipe.to(device)
    # image and mask_image should be PIL images.
    # The mask structure is white for inpainting and black for keeping as s
    ori_img, mask_img, o_w, o_h = img_preprocess_sd_inpaint(
        image,
        mask
    )
    image = pipe(prompt=prompt, image=ori_img, mask_image=mask_img).images[0]
    image = np.array(image)
    image = Image.fromarray(image)
    image = image.resize((o_w, o_h))
    base_path = app_abs_path()
    out_img_path = f'{base_path}/frontend/edit-anything/tmp/img/out/{out_img}.png'
    image.save(out_img_path)

    return f'{out_img}.png'


def inference_sd_generate(prompt: str, width: int, height: int, device: str, out_img: str):
    # todo
    model_id = "stabilityai/stable-diffusion-2"
    # Use the Euler scheduler here instead
    scheduler = EulerDiscreteScheduler.from_pretrained(model_id, subfolder="scheduler")
    pipe = StableDiffusionPipeline.from_pretrained(model_id, scheduler=scheduler, torch_dtype=torch.float16)
    pipe = pipe.to(device)
    image = pipe(prompt).images[0]
    base_path = app_abs_path()
    out_img_path = f'{base_path}/frontend/edit-anything/tmp/img/out/{out_img}.png'
    image.save(out_img_path)
    return f'{out_img}.png'


if __name__ == '__main__':
    pass



import numpy as np
from PIL import Image
import base64
from io import BytesIO
from fastmodel.backend.constant.biz_constants import app_abs_path


def img_base64_to_file(img_base64: str, img_name: str):
    img_decode_data = base64.b64decode(img_base64.split(",")[1])
    img = Image.open(BytesIO(img_decode_data))
    base_path = app_abs_path()
    img_path = f'{base_path}/frontend/edit-anything/tmp/img/in/{img_name}.png'
    img.save(img_path)
    return img_path


def img_preprocess_sd_inpaint(img_path: str, mask_path: str):
    ori_img = Image.open(img_path)
    if ori_img is None:
        print(f'failed to load origin image: {img_path}')
        return
    o_w, o_h = ori_img.size

    mask_img = Image.open(mask_path)
    if mask_img is None:
        print(f'failed to load origin image: {mask_path}')
        return
    m_w, m_h = mask_img.size

    assert (o_w, o_h) == (m_w, m_h), \
        f'shape of the origin image and its mask image should be in same ' \
        f'shape, but got {(o_w, o_h)} vs {(m_w, m_h)}'

    # resize image and its mask to target shape (512, 512)
    if o_w != 512 or o_h != 512:
        ori_img = ori_img.resize((512, 512))
        mask_img = mask_img.resize((512, 512))

    # convert to RGB if image's format is RGBA
    if ori_img.mode == 'RGBA':
        rgb_image = Image.new('RGB', ori_img.size, (255, 255, 255))
        rgb_image.paste(ori_img, mask=ori_img.split()[3])
        ori_img = rgb_image

    ori_img = np.array(ori_img)

    # convert mask_img to grayscale
    mask_img = mask_img.convert('L')
    # make mask binary
    mask_img = np.array(mask_img)
    mask_img[mask_img < 128] = 0
    mask_img[mask_img > 128] = 255
    return Image.fromarray(ori_img), Image.fromarray(mask_img), o_w, o_h


if __name__ == '__main__':
    # # img_base64 = ''
    # # img_base64_to_file(img_base64=img_base64)
    # org_img_path = '/tmp/img/ori_img.png'
    # mask_img_path = '/tmp/img/mask_img.png'
    # ori_img, mask_img = img_preprocess_sd_inpaint(org_img_path, mask_img_path)
    # print(ori_img, mask_img)
    pass



import os
from fastapi import FastAPI
import uvicorn
from fastapi.responses import FileResponse
from starlette.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastmodel.backend.domain.dto.req_classes import SdInpaintReqDto, SdGenerateReqDto
from fastmodel.backend.util.str_util import safe_join, generate_random_string
from fastmodel.backend.inference.inference import inference_sd_inpaint, inference_sd_generate
from fastmodel.backend.util.img_util import img_base64_to_file
from fastmodel.backend.constant.biz_constants import app_abs_path
import argparse

base_dir = app_abs_path()  # 获取当前脚本所在的目录
print(base_dir)
static_resource_dir = os.path.join(base_dir, "frontend/edit-anything")  # 静态文件所在的目录
CSS_PATH_LIB = f"{static_resource_dir}/assets/css"
IMG_PATH_LIB = f"{static_resource_dir}/assets/img"
JS_PATH_LIB = f"{static_resource_dir}/js"
OUT_IMG_PATH_LIB = f"{base_dir}/frontend/edit-anything/tmp/img/out"

app = FastAPI()
app.mount("/static", StaticFiles(directory=static_resource_dir), name="static")


@app.get("/", response_class=HTMLResponse)
@app.get("/home", response_class=HTMLResponse)
async def server():
    return FileResponse(f"{static_resource_dir}/index.html")


@app.get("/assets/css/{path:path}")
async def build_resource(path: str):
    build_file = safe_join(CSS_PATH_LIB, path)
    return FileResponse(build_file)


@app.get("/js/{path:path}")
async def build_resource(path: str):
    js_file = safe_join(JS_PATH_LIB, path)
    return FileResponse(js_file)


@app.post("/gw/edit-anything/api/v1/bff/sd/inpaint")
async def sd_inpaint(body: SdInpaintReqDto):
    image_base64 = body.image
    mask_base64 = body.mask
    image = img_base64_to_file(image_base64, generate_random_string(6))
    mask = img_base64_to_file(mask_base64, generate_random_string(6))
    prompt = body.prompt
    args = parse_args()
    device = args.device
    inpaint_img_path = inference_sd_inpaint(image, mask, prompt, device, generate_random_string(7))

    return {
        "msg": "ok",
        "msgCode": "10000",
        "success": True,
        "total": 1,
        "data": {
            "imgBase64": None,
            "imgUrl": inpaint_img_path
        }
    }


@app.post('/gw/edit-anything/api/v1/bff/sd/generate')
async def sd_generate(body: SdGenerateReqDto):
    prompt = body.prompt
    width = body.width
    height = body.height
    args = parse_args()
    device = args.device
    generate_img_path = inference_sd_generate(prompt, width, height, device, generate_random_string(9))
    return {
        "msg": "ok",
        "msgCode": "10000",
        "success": True,
        "total": 1,
        "data": {
            "imgBase64": None,
            "imgUrl": generate_img_path
        }
    }


@app.get("/favicon.ico")
async def favicon():
    ico_file = safe_join(IMG_PATH_LIB, 'favicon.ico')
    return FileResponse(ico_file)


@app.get("/{path:path}")
async def img_outputs(path: str):
    img_file = safe_join(OUT_IMG_PATH_LIB, path)
    return FileResponse(img_file)


def parse_args():
    parser = argparse.ArgumentParser(description='Light Up Your Imagination From Edit-Anything')
    parser.add_argument(
        '-H',
        '--host',
        type=str,
        required=False,
        default='0.0.0.0',
        help='web server url, default=0.0.0.0'
    )
    parser.add_argument(
        '-P',
        '--port',
        type=int,
        required=False,
        default=9911,
        help='web server port, default=9911'
    )
    parser.add_argument(
        '-D',
        '--device',
        type=str,
        required=False,
        default='cpu',
        help='device type: cpu or cuda'
    )
    return parser.parse_args()


# init:
# 1. create tmp dirs to store temp files
def init():
    to_create_dirs = [
        f'{base_dir}/tmp/img/in',
        f'{base_dir}/tmp/img/out'
    ]
    for item in to_create_dirs:
        if not os.path.exists(item):
            os.makedirs(item)


def main():
    init()
    args = parse_args()
    uvicorn.run(app, host=args.host, port=args.port, timeout_keep_alive=600)


if __name__ == "__main__":
    main()



from fastmodel.backend.exception.exceptions import HTTPException
import os
from typing import List
import random
import string

def safe_join(directory: str, path: str) -> str:
    """Safely path to a base directory to avoid escaping the base directory.
    Borrowed from: werkzeug.security.safe_join"""
    _os_alt_seps: List[str] = [
        sep for sep in [os.path.sep, os.path.altsep] if sep is not None and sep != "/"
    ]

    if path == "":
        raise HTTPException(400)

    filename = os.path.normpath(path)
    fullpath = os.path.join(directory, filename)
    if (
            any(sep in filename for sep in _os_alt_seps)
            or os.path.isabs(filename)
            or filename == ".."
            or filename.startswith("../")
            or os.path.isdir(fullpath)
    ):
        raise HTTPException(403)

    if not os.path.exists(fullpath):
        raise HTTPException(404, "File not found")

    return fullpath


def generate_random_string(length: int):
    letters = string.ascii_lowercase
    result_str = ''.join(random.choice(letters) for i in range(length))
    return result_str


if __name__ == '__main__':
    print(generate_random_string(7))

    # safe_join()



from pydantic import BaseModel
from typing import Optional


class SdInpaintReqDto(BaseModel):
    image: str
    mask: str
    prompt: str
    device: Optional[str] = 'cpu'


class SdGenerateReqDto(BaseModel):
    prompt: str
    height: Optional[int] = 768
    width: Optional[int] = 1024
